/*! Tripetto Runner Foundation 6.0.5-4 - Copyright (C) 2021 Tripetto B.V. - All Rights Reserved */

/// <reference path="marblecore.matrix.d.ts" />
/// <reference path="marblecore.functional.d.ts" />
/// <reference path="marblecore.hookup.d.ts" />
/// <reference path="marblecore.serializer.d.ts" />

declare module "@tripetto/slots" {
    /** Explicit exports */
    export { Slots } from "@tripetto/slots/lib/slots";
    export { Slot } from "@tripetto/slots/lib/slot";
    export { ISlot } from "@tripetto/slots/lib/interface";
    export { ISlotProperties } from "@tripetto/slots/lib/properties";
    export { ISlotType } from "@tripetto/slots/lib/type";
    /** Types */
    export { Kinds } from "@tripetto/slots/lib/kinds";
    export { TPipeable } from "@tripetto/slots/lib/pipeable";
    export { Transformations } from "@tripetto/slots/lib/types/text/transformations";
    export { Precision } from "@tripetto/slots/lib/types/date/precision";
    /** Events */
    export {
        ISlotPropertyEvent,
        ISlotRenameEvent,
        ISlotRequiredEvent,
    } from "@tripetto/slots/lib/events";
    /** Decorators */
    export { slot, property } from "@tripetto/slots/lib/decorators";
    /** Built-in types */
    export { BooleanSlot as Boolean } from "@tripetto/slots/lib/types/boolean/boolean";
    export { DateSlot as Date } from "@tripetto/slots/lib/types/date/date";
    export { NumberSlot as Number } from "@tripetto/slots/lib/types/number/number";
    export { Numeric } from "@tripetto/slots/lib/types/numeric/numeric";
    export { StringSlot as String } from "@tripetto/slots/lib/types/string/string";
    export { Text } from "@tripetto/slots/lib/types/text/text";
}

declare module "@tripetto/slots/lib/slots" {
    import { IDimensionChangeEvent, Matrix } from "@marblecore/matrix";
    import { ISlot } from "@tripetto/slots/lib/interface";
    import { ISlotProperties } from "@tripetto/slots/lib/properties";
    import { ISlotType } from "@tripetto/slots/lib/type";
    import { Kinds } from "@tripetto/slots/lib/kinds";
    import { Slot } from "@tripetto/slots/lib/slot";
    import { TSlotEvents } from "@tripetto/slots/lib/events";
    export class Slots extends Matrix<Slot, TSlotEvents> {
        /**
         * Registers a slot type.
         * @param pType Specifies the slot instance constructor.
         * @param sType Specifies the slot type.
         */
        static register<T extends Slot>(
            pType: ISlotType<T>,
            sType: string
        ): void;
        /**
         * Retrieves the slot constructor of the specified type.
         * @param sType Specifies the slot type.
         * @return Returns the slot constructor or `undefined` if the slot type is not
         * available.
         */
        static ofType<T extends Slot>(sType: string): ISlotType<T> | undefined;
        /** Constructs a new slots instance. */
        constructor(pOrigin?: { readonly id: string });
        /** Retrieves the slots id. */
        get id(): string | undefined;
        /** Retrieves if the slots are being deserialized. */
        get isDeserializing(): boolean;
        /** Retrieves if multiple pipeable slots are present. */
        get hasMultiplePipeable(): boolean;
        /**
         * Fired when the amount of slots is changed.
         * @param pEvent Specifies the event payload.
         * @event
         */
        protected onDimensionChange(pEvent: IDimensionChangeEvent<this>): void;
        /**
         * Creates a new meta slot.
         * @param pProperties Specifies the properties for the slot.
         * @return Returns a reference to the slot.
         */
        meta<T extends Slot>(pProperties: ISlotProperties<T>): T;
        /**
         * Creates a new static slot.
         * @param pProperties Specifies the properties for the slot.
         * @return Returns a reference to the slot.
         */
        static<T extends Slot>(pProperties: ISlotProperties<T>): T;
        /**
         * Creates a new dynamic slot.
         * @param pProperties Specifies the properties for the slot.
         * @return Returns a reference to the slot.
         */
        dynamic<T extends Slot>(pProperties: ISlotProperties<T>): T;
        /**
         * Creates a new feature slot.
         * @param pProperties Specifies the properties for the slot.
         * @return Returns a reference to the slot.
         */
        feature<T extends Slot>(pProperties: ISlotProperties<T>): T;
        /**
         * Selects a slot.
         * @param sReferenceOrId Reference or identifier of the slot.
         * @param sKind Specifies the optional kind of the slot.
         * @return Returns the slot or `undefined` if no slot is found.
         */
        select<T extends Slot>(
            sReferenceOrId: string,
            sKind?: Kinds
        ): T | undefined;
        /**
         * Selects a slot by the specified identifier.
         * @param sId Specifies the slot or pipe identifier to search for.
         * @return Returns the slot or `undefined` if no slot was found.
         */
        selectByIdentifier<T extends Slot>(sId: string): T | undefined;
        /**
         * Deprecates a slot.
         * @param sReferenceOrId Reference or identifier of the slot.
         * @param sKind Specifies the optional kind of the slot.
         * @return Returns the slot or `undefined` if no slot is found.
         */
        deprecate<T extends Slot>(
            sReferenceOrId: string,
            sKind?: Kinds
        ): T | undefined;
        /**
         * Deletes a slot.
         * @param sReferenceOrId Reference or identifier of the slot.
         * @param sKind Specifies the optional kind of the slot.
         * @return Returns the slot or `undefined` if no slot is found.
         */
        delete<T extends Slot>(
            sReferenceOrId: string,
            sKind?: Kinds
        ): T | undefined;
        /**
         * Removes all the slots from the collection except the supplied slots.
         * @param sKind Specifies the kind of slots to remove (or `undefined` to remove all).
         * @param pExclude Specifies the slots to exclude. Those are kept in the slots collection.
         */
        clean(sKind?: Kinds, ...pExclude: (Slot | undefined)[]): void;
        /**
         * Sorts the slots.
         * @return Returns `true` if the order of the slots is changed.
         */
        sort(): boolean;
        /**
         * Serializes the slots to an array.
         * @return Returns the slots array.
         */
        serialize(): ISlot[];
        /**
         * Deserializes slots.
         * @param pSlots Specifies the slots array.
         * @return Returns a reference to the instance.
         */
        deserialize(pSlots: ISlot[]): this;
        /** Destroys the slots instance. */
        destroy(): void;
    }
}

declare module "@tripetto/slots/lib/slot" {
    /** Dependencies */
    import { TAny } from "@marblecore/functional";
    import { IItemPopEvent, Item } from "@marblecore/matrix";
    import { HookProps } from "@marblecore/hookup";
    import { TSerializeTypes } from "@marblecore/serializer";
    import { Slots } from "@tripetto/slots/lib/slots";
    import { TSlotEvents } from "@tripetto/slots/lib/events";
    import { ISlot } from "@tripetto/slots/lib/interface";
    import { ISlotType } from "@tripetto/slots/lib/type";
    import { Kinds } from "@tripetto/slots/lib/kinds";
    import { TPipeable } from "@tripetto/slots/lib/pipeable";
    export abstract class Slot<
        T extends TSerializeTypes = TSerializeTypes
    > extends Item<TSlotEvents> {
        /** Contains the static type identifier. */
        static readonly type: string;
        /** Contains the label of the slot. */
        label?: string;
        /** Contains the name of the slot. */
        name?: string;
        /** Contains the placeholder of the slot. */
        placeholder?: string;
        /** Contains an alias for the slot. */
        alias?: string;
        /** Contains if this slot is required. */
        required?: boolean;
        /** Contains if this slot is pipeable. */
        pipeable?: TPipeable;
        /** Contains the default slot data value. */
        default?: T;
        /** Contains if this slot is exportable. */
        exportable?: boolean;
        /** Contains if this slot is actionable. */
        actionable?: boolean;
        /** Contains if this slot is protected. */
        protected?: boolean;
        /**
         * Creates a new slot instance.
         * @param pType Reference to the type constructor.
         * @param sKind Specifies the kind.
         * @param sReference Specifies the reference for the slot.
         * @param nSequence Specifies the sequence number.
         * @param sPipe Specifies the pipe identifier of the slot.
         * @param pExchangables Specifies the exchangeable slot properties.
         * @param pSlotsOrId Contains a reference to the slots or the id for the slot.
         */
        constructor(
            pType: ISlotType,
            sKind: Kinds,
            sReference: string,
            nSequence?: number,
            sPipe?: string,
            pExchangables?: string[],
            pSlotsOrId?: Slots | string
        );
        /** Retrieves a reference to the slots. */
        get slots(): Slots | undefined;
        /** Retrieves the identifier of the slot. */
        get id(): string;
        /** Retrieves the slot type identifier. */
        get type(): string;
        /** Retrieves the slot kind. */
        get kind(): Kinds;
        /** Retrieves the sequence number. */
        get sequence(): number | undefined;
        /** Sets the sequence number. */
        set sequence(nSequence: number | undefined);
        /** Retrieves the slot reference. */
        get reference(): string;
        /** Retrieves the exchangeable slot properties. */
        get exchangeables(): {
            property: string;
            value: TAny;
        }[];
        /**
         * Fired when the slot is popped.
         * @param pEvent Specifies the event payload.
         * @event
         */
        protected onPop(pEvent: HookProps<IItemPopEvent<this>>): void;
        /**
         * Emits the `OnSlotProperty` event.
         * @param sProperty Specifies the changed property.
         * @event
         */
        propertyChange(sProperty: keyof this): void;
        /**
         * Serializes a slot.
         * @return Returns a reference to the serialized slot.
         */
        serialize(): ISlot<T>;
        /**
         * Deserializes to a slot instance.
         * @param pSlot Contains the serialized slot.
         * @return Returns a reference to the instance.
         */
        deserialize(pSlot: ISlot<T>): this;
        /**
         * Deprecates a slot.
         * @return Returns a reference to the instance.
         */
        deprecate(): this;
        /**
         * Deletes a slot.
         * @return Returns a reference to the instance.
         */
        delete(): this;
        /** Retrieves the pipe identifier for the slot. */
        pipe(pOrigin?: { readonly id: string }): string;
        /**
         * Retrieves the legacy pipe identifier for the slot.
         * @deprecated
         */
        legacyPipe(pOrigin?: { readonly id: string }): string | undefined;
        /**
         * Verifies if the supplied identifier is a pipe.
         * @param sId Specifies the identifier to check.
         * @return Returns `true` if the identifier is a pipe.
         */
        isPipe(sId: string | undefined): boolean;
        /** Specifies the exchangeable slot properties. */
        exchange(pExchangables: string[] | undefined): void;
        /**
         * Converts the supplied data to a valid value.
         * @param pData Specifies the data.
         * @return Returns the data value.
         */
        abstract toValue(pData: TSerializeTypes): T;
        /**
         * Converts the supplied data to a valid string.
         * @param pData Specifies the data.
         * @return Returns the data as a string.
         */
        abstract toString(pData: TSerializeTypes): string;
    }
}

declare module "@tripetto/slots/lib/interface" {
    /** Dependencies */
    import { ISerialize, TSerializeTypes } from "@marblecore/serializer";
    import { ISlot as ISlotProps } from "@tripetto/slots/lib/properties";
    export interface ISlot<T extends TSerializeTypes = TSerializeTypes>
        extends ISlotProps<T>,
            ISerialize {}
}

declare module "@tripetto/slots/lib/properties" {
    /** Dependencies */
    import { TSerializeTypes } from "@marblecore/serializer";
    import { ISlotType } from "@tripetto/slots/lib/type";
    import { Kinds } from "@tripetto/slots/lib/kinds";
    import { Slot } from "@tripetto/slots/lib/slot";
    import { TPipeable } from "@tripetto/slots/lib/pipeable";
    export interface ISlot<T extends TSerializeTypes = TSerializeTypes> {
        /** Contains the id for the serialized slot. */
        readonly id: string;
        /** Contains the slot type. */
        readonly type: string;
        /** Contains the slot kind. */
        readonly kind: Kinds;
        /** Contains the slot reference. */
        readonly reference: string;
        /** Contains the slot sequence number. */
        readonly sequence?: number;
        /** Contains the label for the slot. */
        readonly label?: string;
        /** Contains the slot name. */
        readonly name?: string;
        /** Contains the slot placeholder. */
        readonly placeholder?: string;
        /** Contains the slot alias. */
        readonly alias?: string;
        /** Contains if the slot is required. */
        readonly required?: boolean;
        /** Contains if the slot is pipeable. */
        readonly pipeable?: TPipeable;
        /** Contains the default slot value. */
        readonly default?: T;
        /** Contains if this slot is exportable. */
        readonly exportable?: boolean;
        /** Contains if the slot is actionable. */
        readonly actionable?: boolean;
        /** Contains if the slot is protected. */
        readonly protected?: boolean;
    }
    export interface ISlotProperties<T extends Slot>
        extends Partial<
            Pick<T, Exclude<keyof ISlot, "id" | "type" | "kind" | "reference">>
        > {
        /** Contains the slot type. */
        readonly type: ISlotType<T>;
        /** Contains the slot reference. */
        readonly reference: string;
        /** Contains the label for the slot. */
        readonly label: string;
        /** Defines the pipe identifier of the slot. */
        readonly pipe?: string;
        /** Specifies the exchangeable slot properties. */
        readonly exchange?: (keyof T)[];
    }
}

declare module "@tripetto/slots/lib/type" {
    /** Dependencies */
    import { Slot } from "@tripetto/slots/lib/slot";
    import { Slots } from "@tripetto/slots/lib/slots";
    import { Kinds } from "@tripetto/slots/lib/kinds";
    export interface ISlotType<T extends Slot = Slot> {
        /** Contains the slot type identifier. */
        readonly type: string;
        /**
         * Creates a new slot instance.
         * @param nSlotsId Contains the slots identifier.
         * @param pType Reference to the type constructor.
         * @param sKind Specifies the kind.
         * @param sReference Specifies the reference.
         * @param nSequence Specifies the sequence number.
         * @param sPipe Specifies the pipe identifier of the slot.
         * @param pExchangables Specifies the exchangeable slot properties.
         * @param pSlotsOrId Contains a reference to the slots or the id for the slot.
         */
        new (
            pType: ISlotType,
            sKind: Kinds,
            sReference: string,
            nSequence?: number,
            sPipe?: string,
            pExchangables?: (keyof T)[],
            pSlotsOrId?: Slots | string
        ): T;
    }
}

declare module "@tripetto/slots/lib/kinds" {
    /** Specifies the slot kinds. */
    export type Kinds = "meta" | "static" | "dynamic" | "feature";
    /**
     * Retrieves the static order index for the supplied kind.
     * @param sKind Specifies the kind.
     * @return Returns the order index.
     */
    export function kindOrderIndexFor(sKind: Kinds): number;
}

declare module "@tripetto/slots/lib/pipeable" {
    export type TPipeable =
        | boolean
        | {
              /** Optional name for the pipe. */
              readonly pipe?: string;
              /** Optional localized label for the pipe. */
              readonly label?: string;
              /** Optional alias for the pipe. */
              readonly alias?: string;
              /** Specifies the field or content that should be used as the pipe string (defaults to `value` which is the current value of the slot). */
              readonly content?:
                  | "value"
                  | "label"
                  | "name"
                  | {
                        /** Contains the content as a string without any markup or variables. */
                        readonly string: string;
                        /** Contains the content as text with support for variables. */
                        readonly text?: string;
                        /** Contains markdown content with support for basic formatting, hyperlinks and variables. */
                        readonly markdown?: string;
                    };
              /**
               * Specifies the name of a legacy pipe. Only here for backwards compatibility.
               * @deprecated
               */
              readonly legacy?: string;
          };
}

declare module "@tripetto/slots/lib/types/text/transformations" {
    export type Transformations =
        | "none"
        | "capitalize"
        | "capitalize-words"
        | "capitalize-sentences"
        | "uppercase"
        | "lowercase";
}

declare module "@tripetto/slots/lib/types/date/precision" {
    export type Precision =
        | "years"
        | "months"
        | "days"
        | "hours"
        | "minutes"
        | "seconds"
        | "milliseconds";
}

declare module "@tripetto/slots/lib/events" {
    /** Dependencies */
    import { IHookPayload, THooks } from "@marblecore/hookup";
    import { Slot } from "@tripetto/slots/lib/slot";
    export type TSlotEvents =
        | "OnSlotRename"
        | "OnSlotRequired"
        | "OnSlotProperty";
    export interface ISlotEvent<H extends THooks, T extends Slot>
        extends IHookPayload<H> {
        /** Reference to the slot. */
        readonly slot: T;
    }
    export interface ISlotRenameEvent<T extends Slot>
        extends ISlotEvent<"OnSlotRename", T> {}
    export interface ISlotRequiredEvent<T extends Slot>
        extends ISlotEvent<"OnSlotRequired", T> {}
    export interface ISlotPropertyEvent<T extends Slot>
        extends ISlotEvent<"OnSlotProperty", T> {
        /** Specifies the changed property. */
        readonly property: keyof T;
    }
}

declare module "@tripetto/slots/lib/decorators" {
    /** Dependencies */
    import { ISlotType } from "@tripetto/slots/lib/type";
    import { Slot } from "@tripetto/slots/lib/slot";
    /**
     * Defines a slot type.
     * @param sType Specifies the slot type name.
     */
    export function slot<T extends Slot>(
        sType: string
    ): (pType: ISlotType<T>) => void;
    /**
     * Marks a property as a slot property.
     * @param pTarget Reference to the slot.
     * @param sProperty Specifies the slot property.
     * @param pDescriptor Reference to the property descriptor.
     */
    export function property(
        pTarget: Slot,
        sProperty: string,
        pDescriptor?: PropertyDescriptor
    ): void;
}

declare module "@tripetto/slots/lib/types/boolean/boolean" {
    import { TSerializeTypes } from "@marblecore/serializer";
    import { Slot } from "@tripetto/slots/lib/slot";
    export class BooleanSlot extends Slot<boolean> {
        /** Contains the string label for false values. */
        labelForFalse?: string;
        /** Contains the string label for true values. */
        labelForTrue?: string;
        /**
         * Converts the supplied data to a valid boolean value.
         * @param pData Specifies the data.
         * @return Returns the boolean value.
         */
        toValue(pData: TSerializeTypes): boolean;
        /**
         * Converts the supplied data to a valid string.
         * @param pData Specifies the data.
         * @return Returns the data as a string.
         */
        toString(pData: TSerializeTypes): string;
    }
}

declare module "@tripetto/slots/lib/types/date/date" {
    import { TSerializeTypes } from "@marblecore/serializer";
    import { Slot } from "@tripetto/slots/lib/slot";
    import { Precision } from "@tripetto/slots/lib/types/date/precision";
    export class DateSlot extends Slot<number> {
        /** Specifies the date/time precision (defaults to `milliseconds`). */
        precision?: Precision;
        /** Contains the minimum date. */
        minimum?: Date | number | true;
        /** Contains the maximum date. */
        maximum?: Date | number | true;
        /** Retrieves if the slot supports time. */
        get supportsTime(): boolean;
        /**
         * Converts the supplied data to a date value.
         * @param pData Specifies the data.
         * @param sPrecision Specifies the precision to use.
         * @return Returns the date value.
         */
        toValue(pData: TSerializeTypes | Date, sPrecision?: Precision): number;
        /**
         * Converts the supplied data to a valid date object.
         * @param pData Specifies the data.
         * @param sPrecision Specifies the precision to use.
         * @return Returns the data as a date object.
         */
        toDate(pData: TSerializeTypes | Date, sPrecision?: Precision): Date;
        /**
         * Converts the supplied data to a valid string.
         * @param pData Specifies the data.
         * @param sPrecision Specifies the precision to use.
         * @return Returns the data as a string.
         */
        toString(pData: TSerializeTypes | Date, sPrecision?: Precision): string;
    }
}

declare module "@tripetto/slots/lib/types/number/number" {
    import { TSerializeTypes } from "@marblecore/serializer";
    import { Slot } from "@tripetto/slots/lib/slot";
    export class NumberSlot extends Slot<number> {
        /**
         * Converts the supplied data to a valid number value.
         * @param pData Specifies the data.
         * @return Returns the number value.
         */
        toValue(pData: TSerializeTypes): number;
        /**
         * Converts the supplied data to a valid string.
         * @param pData Specifies the data.
         * @param fnFormatter Optional formatter function used when no separator, decimal or minus sign is specified.
         * @return Returns the data as a string.
         */
        toString(
            pData: TSerializeTypes,
            fnFormatter?: (nValue: number | string) => string
        ): string;
    }
}

declare module "@tripetto/slots/lib/types/numeric/numeric" {
    import { TSerializeTypes } from "@marblecore/serializer";
    import { Slot } from "@tripetto/slots/lib/slot";
    export class Numeric extends Slot<number> {
        /** Contains the minimal value. */
        minimum?: number;
        /** Contains the maximum value. */
        maximum?: number;
        /** Contains the thousands separtor sign. */
        separator?: string;
        /** Contains the decimal sign. */
        decimal?: string;
        /** Contains the minus sign. */
        minus?: string;
        /** Contains the prefix. */
        prefix?: string;
        /** Contains the plural prefix. */
        prefixPlural?: string;
        /** Contains the suffix. */
        suffix?: string;
        /** Contains the plural suffix. */
        suffixPlural?: string;
        /** Contains the number of digits. */
        digits?: number;
        /** Retrieves the numeric precision. */
        get precision(): number | undefined;
        /** Sets the numeric precision. */
        set precision(nPrecision: number | undefined);
        /**
         * Converts the supplied data to a valid numeric value.
         * @param pData Specifies the data.
         * @return Returns the numeric value.
         */
        toValue(pData: TSerializeTypes): number;
        /**
         * Formats a string.
         * @param sNumber Specifies the number to format.
         * @param bPlural Specifies if the plural prefix and suffix should be used.
         * @return Returns the formatted string.
         */
        formatString(sNumber: string, bPlural: boolean): string;
        /**
         * Converts the supplied data to a valid string.
         * @param pData Specifies the data.
         * @param fnFormatter Optional formatter function used when no separator, decimal or minus sign is specified.
         * @return Returns the data as a string.
         */
        toString(
            pData: TSerializeTypes,
            fnFormatter?: (
                nValue: number | string,
                nPrecision?: number | "auto"
            ) => string
        ): string;
    }
}

declare module "@tripetto/slots/lib/types/string/string" {
    import { TSerializeTypes } from "@marblecore/serializer";
    import { Slot } from "@tripetto/slots/lib/slot";
    export class StringSlot extends Slot<string> {
        /**
         * Converts the supplied data to a valid string value.
         * @param pData Specifies the data.
         * @return Returns the string value.
         */
        toValue(pData: TSerializeTypes): string;
        /**
         * Converts the supplied data to a valid string.
         * @param pData Specifies the data.
         * @return Returns the data as a string.
         */
        toString(pData: TSerializeTypes): string;
    }
}

declare module "@tripetto/slots/lib/types/text/text" {
    import { TSerializeTypes } from "@marblecore/serializer";
    import { Slot } from "@tripetto/slots/lib/slot";
    import { Transformations } from "@tripetto/slots/lib/types/text/transformations";
    export class Text extends Slot<string> {
        /** Retrieves the maximum text length. */
        get maxLength(): number | undefined;
        /** Sets the maximum text length. */
        set maxLength(nMaxLength: number | undefined);
        /** Specifies the transformation to apply. */
        transformation?: Transformations;
        /**
         * Converts the supplied data to a valid text value.
         * @param pData Specifies the data.
         * @return Returns the text value.
         */
        toValue(pData: TSerializeTypes): string;
        /**
         * Converts the supplied data to a valid string.
         * @param pData Specifies the data.
         * @return Returns the data as a string.
         */
        toString(pData: TSerializeTypes): string;
    }
}
